import Component from '@ember/component'
import { htmlSafe } from '@ember/string'
import { get, set } from '@ember/object'

import layout from '../templates/components/en-dropdown'
import DocumentEvent from 'en-select/mixins/document-event'
import KeyMap from 'en-select/utils/keymap'

export default Component.extend(DocumentEvent, {
  layout,

  classNames: ['dropdown-container'],
  classNameBindings: ['isFixed:is-fixed', 'isDisabled:isDisabled'],
  documentEvents: ['click'],

  isOpen: false,
  isFixed: false,
  optionsStyle: htmlSafe(''),

  onDocumentClick(e) {
    if (!get(this, 'isOpen')) return
    let content = this.element.querySelector('.dropdown-content')

    if (
      !content ||
      !(content instanceof HTMLElement) ||
      content.contains(e.target)
    ) {
      return
    }

    this.closeDropdown()
  },

  keyDown(e) {
    const { which, keyCode } = e
    const key = which || keyCode

    if (key === KeyMap.esc) {
      this.closeDropdown()
    }
  },

  closeDropdown() {
    set(this, 'isOpen', false)
    if (this.attrs['on-close']) this.attrs['on-close']()
  },

  openDropdown() {
    set(this, 'isOpen', true)
    if (this.attrs['on-open']) this.attrs['on-open']()
  },
})
