import Component from '@ember/component'
import { htmlSafe } from '@ember/string'
import { get, set } from '@ember/object'

import layout from '../../templates/components/en-dropdown/content'
import DocumentEvent from 'en-select/mixins/document-event'

export default Component.extend(DocumentEvent, {
  layout,
  classNames: ['dropdown-content'],
  classNameBindings: [
    'isOpen:is-open',
    'isDisabled:is-disabled',
    'positionFromTop:from-top',
    'positionFromRight:from-right',
  ],
  attributeBindings: ['style'],
  documentEvents: ['scroll', 'resize'],

  style: htmlSafe(''),

  isOpen: false,
  isDisabled: false,

  positionFromTop: false,
  positionFromRight: false,

  didReceiveAttrs() {
    if (get(this, 'isOpen')) this._repositionDropdown()
  },

  onDocumentScroll() {
    this._repositionDropdown()
  },

  onDocumentResize() {
    this._repositionDropdown()
  },

  _repositionDropdown() {
    let { element: el } = this
    if (!el || !(el instanceof HTMLElement)) return

    let { right, top } = el.getBoundingClientRect()

    let width = window.innerWidth
    let height = window.innerHeight

    if (width - right < 150) {
      set(this, 'positionFromRight', true)
    }

    if (height - top < 150) {
      set(this, 'positionFromTop', true)
    }
  },
})
