import Component from '@ember/component'
import { get } from '@ember/object'
import { A } from '@ember/array'
import { isEmpty } from '@ember/utils'

import layout from '../templates/components/en-scrollable-list'
import KeyMap from 'en-select/utils/keymap'

export default Component.extend({
  layout,

  classNames: ['en-scrollable-list'],

  options: A([]),
  focusedOption: null,

  keyUp(event) {
    const { which, keyCode } = event
    const key = which || keyCode

    switch (key) {
      case KeyMap.up:
        this._move('up', event)
        break

      case KeyMap.down:
        this._move('down', event)
        break

      case KeyMap.enter:
        this.attrs['on-enter'](get(this, 'focusedOption'), event)
        break

      case KeyMap.esc:
        this.attrs['on-esc'](event)
        break

      default:
        break
    }
  },

  _findIndices(options, focused) {
    let indexOfFocus = options.indexOf(focused)

    let lastIsFocused = indexOfFocus === options.length - 1
    let firstIsFocused = indexOfFocus === 0
    let nothingIsFocused = isEmpty(focused)

    return {
      indexOfFocus,
      firstIsFocused,
      lastIsFocused,
      nothingIsFocused,
    }
  },

  _findNextFocus(direction, options, focused) {
    let nextFocus

    let {
      indexOfFocus,
      firstIsFocused,
      lastIsFocused,
      nothingIsFocused,
    } = this._findIndices(options, focused)

    let up = direction === 'up'
    let down = direction === 'down'

    /*
     * In case the first element is focused, and the direction is
     * up, we move to the last element.
    */

    if ((firstIsFocused || nothingIsFocused) && up) {
      nextFocus = options[options.length - 1]
    } else if ((lastIsFocused || nothingIsFocused) && down) {
      nextFocus = options[0]
    } else {
      let delta

      if (direction === 'up') {
        delta = -1
      } else if (direction === 'down') {
        delta = 1
      }

      nextFocus = options[indexOfFocus + delta]
    }

    return nextFocus
  },

  _move(direction, event) {
    let options = get(this, 'options')
    let focused = get(this, 'focusedOption')

    let nextFocus = this._findNextFocus(direction, options, focused)

    if (isEmpty(nextFocus)) {
      return
    }

    this.attrs['on-focus'](nextFocus, event)
  },
})
