import Mixin from '@ember/object/mixin'
import { get } from '@ember/object'
import { run } from '@ember/runloop'
import { warn } from '@ember/debug'

const capitalize = str => str.charAt(0).toUpperCase() + str.slice(1)

export default Mixin.create({
  documentEvents: [],

  didInsertElement() {
    this._subscribeToEvents()
  },

  _subscribeToEvents() {
    run.scheduleOnce('afterRender', () => {
      get(this, 'documentEvents').forEach(event => {
        const eventHandler = `onDocument${capitalize(event)}`

        if (!this[eventHandler]) {
          warn(
            `[document-event] You are tracking ${event}, but there is no handler. ${eventHandler} was not found`,
          )
          return
        }

        this[eventHandler] = this[eventHandler].bind(this)
        window.addEventListener(event, this[eventHandler], true)
      })
    })
  },

  willDestroyElement() {
    this._teardown()
  },

  _teardown() {
    get(this, 'documentEvents').forEach(event => {
      const eventHandler = `onDocument${capitalize(event)}`
      window.removeEventListener(event, this[eventHandler], true)
    })
  },
})
